@@ -10,6 +10,7 @@ extern crate anyhow;
 pub mod sink;
 pub mod source;
 
+pub mod buffer_gen;
 pub mod state;
 
 pub use sink::output_format::KafkaOutputFormat;
@@ -18,9 +19,11 @@ pub use source::input_format::KafkaInputFormat;
 use std::collections::HashMap;
 
 use rdkafka::ClientConfig;
-use rlink::core::element::BufferReader;
+use rlink::core::element::FnSchema;
 use rlink::core::element::Record;
+use rlink::core::properties::Properties;
 
+use crate::buffer_gen::kafka_message;
 use crate::source::deserializer::{
     DefaultKafkaRecordDeserializer, DefaultKafkaRecordDeserializerBuilder,
     KafkaRecordDeserializerBuilder,
@@ -30,25 +33,18 @@ use crate::state::PartitionOffset;
 pub const BOOTSTRAP_SERVERS: &str = "bootstrap.servers";
 pub const TOPICS: &str = "topics";
 pub const GROUP_ID: &str = "group.id";
+pub const BUFFER_SIZE: &str = "buffer.size";
+
+pub const KAFKA: &str = "kafka";
+pub const REPLAY: &str = "replay";
+pub const OFFSET_BEGIN: &str = "offset.begin";
+pub const OFFSET_END: &str = "offset.end";
+
+pub const INPUT_FORMAT_FN_NAME_DEFAULT: &str = "KafkaInputFormat";
 
 pub const SOURCE_CHANNEL_SIZE: usize = 50000;
 pub const SINK_CHANNEL_SIZE: usize = 50000;
 
-pub static KAFKA_DATA_TYPES: [u8; 6] = [
-    // timestamp
-    rlink::core::element::types::I64,
-    // key
-    rlink::core::element::types::BYTES,
-    // payload
-    rlink::core::element::types::BYTES,
-    // topic
-    rlink::core::element::types::BYTES,
-    // partition
-    rlink::core::element::types::I32,
-    // offset
-    rlink::core::element::types::I64,
-];
-
 pub fn build_kafka_record(
     timestamp: i64,
     key: &[u8],
@@ -57,57 +53,26 @@ pub fn build_kafka_record(
     partition: i32,
     offset: i64,
 ) -> Result<Record, std::io::Error> {
+    let message = kafka_message::Entity {
+        timestamp,
+        key,
+        payload,
+        topic,
+        partition,
+        offset,
+    };
+
     // 36 = 12(len(payload) + len(topic) + len(key)) +
     //      20(len(timestamp) + len(partition) + len(offset)) +
     //      4(place_holder)
     let capacity = payload.len() + topic.len() + key.len() + 36;
     let mut record = Record::with_capacity(capacity);
-    let mut writer = record.as_writer(&KAFKA_DATA_TYPES);
-    writer.set_i64(timestamp)?;
-    writer.set_bytes(key)?;
-    writer.set_bytes(payload)?;
-    writer.set_str(topic)?;
-    writer.set_i32(partition)?;
-    writer.set_i64(offset)?;
+
+    message.to_buffer(record.as_buffer()).unwrap();
 
     Ok(record)
 }
 
-pub struct KafkaRecord<'a, 'b> {
-    reader: BufferReader<'a, 'b>,
-}
-
-impl<'a, 'b> KafkaRecord<'a, 'b> {
-    pub fn new(record: &'a mut Record) -> Self {
-        let reader = record.as_reader(&KAFKA_DATA_TYPES);
-        KafkaRecord { reader }
-    }
-
-    pub fn get_kafka_timestamp(&self) -> Result<i64, std::io::Error> {
-        self.reader.get_i64(0)
-    }
-
-    pub fn get_kafka_key(&self) -> Result<&[u8], std::io::Error> {
-        self.reader.get_bytes(1)
-    }
-
-    pub fn get_kafka_payload(&self) -> Result<&[u8], std::io::Error> {
-        self.reader.get_bytes(2)
-    }
-
-    pub fn get_kafka_topic(&self) -> Result<&str, std::io::Error> {
-        self.reader.get_str(3)
-    }
-
-    pub fn get_kafka_partition(&self) -> Result<i32, std::io::Error> {
-        self.reader.get_i32(4)
-    }
-
-    pub fn get_kafka_offset(&self) -> Result<i64, std::io::Error> {
-        self.reader.get_i64(5)
-    }
-}
-
 pub fn create_output_format(
     conf_map: HashMap<String, String>,
     topic: Option<String>,
@@ -125,6 +90,86 @@ pub fn create_output_format(
     )
 }
 
+pub fn create_input_format(
+    properties: &Properties,
+    fn_name: Option<String>,
+    deserializer_builder: Option<Box<dyn KafkaRecordDeserializerBuilder>>,
+) -> anyhow::Result<KafkaInputFormat> {
+    let fn_name = fn_name.unwrap_or(INPUT_FORMAT_FN_NAME_DEFAULT.to_owned());
+    let source_properties = properties.get_source_properties(fn_name.as_str());
+    let kafka_properties = source_properties.get_sub_properties(KAFKA);
+
+    let mut client_config = ClientConfig::new();
+    let bootstrap_servers = kafka_properties.get_string(BOOTSTRAP_SERVERS)?;
+    let group_id = kafka_properties.get_string(GROUP_ID)?;
+    client_config.set(BOOTSTRAP_SERVERS, bootstrap_servers);
+    client_config.set(GROUP_ID, group_id);
+
+    let topics = kafka_properties.get_string(TOPICS)?;
+    let topics: Vec<String> = serde_json::from_str(topics.as_str())
+        .map_err(|_e| anyhow!("topics '{}' is not array", topics))?;
+
+    let buffer_size = kafka_properties
+        .get_usize(BUFFER_SIZE)
+        .unwrap_or(SOURCE_CHANNEL_SIZE);
+
+    let replay_properties = kafka_properties.get_sub_properties(REPLAY);
+    let offset_range = if replay_properties.as_map().is_empty() {
+        OffsetRange::None
+    } else {
+        let begin_offset = replay_properties.get_string(OFFSET_BEGIN)?;
+        let begin_offset = parse_replay_offset(begin_offset.as_str())?;
+
+        let mut end_offset = None;
+        if let Ok(end_offset_str) = replay_properties.get_string(OFFSET_END) {
+            let end_offset_map = parse_replay_offset(end_offset_str.as_str())?;
+            end_offset = Some(end_offset_map);
+        }
+
+        OffsetRange::Direct {
+            begin_offset,
+            end_offset,
+        }
+    };
+
+    let deserializer_builder = deserializer_builder.unwrap_or_else(|| {
+        let deserializer_builder: Box<dyn KafkaRecordDeserializerBuilder> =
+            Box::new(DefaultKafkaRecordDeserializerBuilder::<
+                DefaultKafkaRecordDeserializer,
+            >::new(FnSchema::from(
+                &kafka_message::FIELD_METADATA,
+            )));
+        deserializer_builder
+    });
+
+    let kafka_input_format = KafkaInputFormat::new(
+        client_config,
+        topics,
+        buffer_size,
+        offset_range,
+        deserializer_builder,
+    );
+    Ok(kafka_input_format)
+}
+
+fn parse_replay_offset(
+    offset_config: &str,
+) -> anyhow::Result<HashMap<String, Vec<PartitionOffset>>> {
+    let offset: HashMap<String, Vec<i64>> = serde_json::from_str(offset_config)
+        .map_err(|_e| anyhow!("offset config is illegal: {}", offset_config))?;
+
+    let mut partition_offset = HashMap::new();
+    for (topic, vec) in offset {
+        let mut offset_vec = Vec::new();
+        for (partition, value) in vec.iter().enumerate() {
+            offset_vec.push(PartitionOffset::new(partition as i32, *value))
+        }
+        partition_offset.insert(topic, offset_vec);
+    }
+
+    Ok(partition_offset)
+}
+
 pub enum OffsetRange {
     None,
     Direct {
@@ -182,7 +227,9 @@ impl InputFormatBuilder {
             let deserializer_builder: Box<dyn KafkaRecordDeserializerBuilder> =
                 Box::new(DefaultKafkaRecordDeserializerBuilder::<
                     DefaultKafkaRecordDeserializer,
-                >::new());
+                >::new(FnSchema::from(
+                    &kafka_message::FIELD_METADATA,
+                )));
 
             deserializer_builder
         });
